from yasi_applets.install import bp
from flask import Flask, request, session, redirect, \
     url_for, render_template, flash, Blueprint, jsonify
from flask import current_app

import subprocess
import threading


status = {
    "status": "idle",  # idle | running | finished
    "output": "",
    "exit_code": None,
    "lines": []
}


@bp.route('/install',methods=['GET', 'POST', 'PUT'])
def install_index():
    """
    The page you'd get if you access the root of
    this app in a browser.
    """
    install_start()
    return render_template('install.html', hide_menu = True, hide_help = True, hide_close = True)


@bp.route('/install-start',methods=['GET', 'POST', 'PUT'])
def install_start():
    """
    Trigger the installation process
    """
    print("The installation process is starting!... in theory at least")
    if status["status"] == "running":
        return jsonify({"status": "already running"}), 409  # Conflict
    threading.Thread(target=run_script).start()
    return jsonify({"status": "started"})
    print("did it finish?")


def run_script():
    global status
    status["status"] = "running"

    process = subprocess.Popen(
        ["dmm-perform-recipe", "/etc/yasi/final-install.yaml"],
        stdout = subprocess.PIPE,
        stderr = subprocess.STDOUT,
        text = True)

    lines = []
    for line in iter(process.stdout.readline,''):
        #lines.append(line.strip())
        #status["output"] = "\n".join(lines)
        status["lines"].append(line.strip())

    process.wait()
    status["status"] = 'finished'
    status["exit_code"] = process.returncode

    print(lines)
    print("status:", status["status"], "code:", status["exit_code"])


@bp.route('/install/status', methods=['GET', 'POST'])
def install_status():
    """
    Update on the status of the installation process.
    """

    # Keep track of the last line posted, so that we can include
    # any new lines not provided to the UI yet
    last_line = int(request.args.get("last_line", 0))
    new_lines = status["lines"][last_line:]

    if not new_lines and status["status"] == "finished":
        return ""  # Empty response = HTMX will stop polling

    lines_html = ""
    if not 'percentage' in status:
        status['percentage'] = "0" 
    for i, line in enumerate(new_lines, start=last_line + 1):
        lines_html += f'<div>{line}</div>'
        if "Running task: " in line:
            status['percentage'] = line.split("Running task:")[1]
        if "Status: " in line:
            status['status_label'] = line.split("Running task:")[1]
            if status['status_label'] == "mount_rootfs":
                status['percentage'] = 50

    return {
        "output": status['output'],
        "status": status['status'],
        "lines": lines_html,
        "percentage": status['percentage'],
        "last_line": f'<input type="hidden" id="last-line" name="last_line" value="{last_line + len(new_lines)}">'
    }


@bp.route('/install/status/percentagediv', methods=['GET'])
def install_percentage_div():
    """
    Provides a new div with the percentage of whatever is happening right now.
    """
    install_status()
    if not 'percentage' in status:
        status['percentage'] = "0" 
    else:
        return '<div class="progress-bar" id="percentagediv" style="width: ' + status['percentage'] + '%;"> </div>'


@bp.route('/install/status/percentage', methods=['GET'])
def install_percentage():
    """
    Provides the percentage of whatever is happening right now.
    """
    install_status()
    if not 'percentage' in status:
        status['percentage'] = "0" 
    return status['percentage']


@bp.route('/install/status/status_label', methods=['GET'])
def install_status_label():
    """
    Provides the status label displayed beneath the progress bar.
    """
    install_status()
    if not 'status_label' in status:
        status['status_label'] = "Initializing..." 
    return status['status_label']


@bp.route('/install/get-slide', methods=['GET'])
def install_get_slide():
    """
    Return slide from installer slideshow.
    """
    if 'status_label' in status:
        if status['status_label'] == "Complete!":
            return render_template("slide_complete.html")
        else:
            return render_template("slide_installing.html")



@bp.route('/install/complete', methods=['GET', 'POST'])
def install_complete():
    """
    Installation complete page.
    """
    return render_template("complete.html")


@bp.route('/install/cancel', methods=["POST"])
def install_cancel():
    """
    Cancel the installation process.
    """
    if status["status"] != "running" or not status["process"]:
        return Response("No running script", status=400)
    process["process"].terminate()
    process["status"] = "cancelled"
    process["exit_code"] = -1
    return Response("Cancelled", status=200)


@bp.route('/settings',methods=['GET', 'POST', 'PUT'])
def settings():
    """
    Manage settings for the installation module.
    """
    print("Settings")



from yasi_applets.welcome import bp
from flask import Flask, request, session, redirect, \
     url_for, render_template, flash, Blueprint
# we use this neat little trick to get config data from the main app
from flask import current_app
import gettext

# Set up Gettext
def set_language(LANG):
    """
    Sets language for this applet
    """
    # TODO: unhardcode this path
    translations = gettext.translation("welcome", '/usr/share/yasi-daemon/yasi_applets/welcome/locales',
                                       fallback=True, languages=[LANG])
    translations.install()
    _ = translations.gettext


@bp.route('/')
def index():
    """
    The page you'd get if you access the root of
    this app in a browser.
    """
    return ("Welcome to System Installer Daemon POC <br />"
            "Version: Unavailable")


@bp.route('/main/help',methods=['GET'])
def main_welcome():
    """
    Manages the main help system.
    """
    return render_template('help.html')


@bp.route('/main/quit',methods=['GET'])
def main_quit():
    """
    Manages the quit dialog.
    """
    return render_template('quit.html')


@bp.route('/main/menu',methods=['GET'])
def main_menu():
    """
    Manages the main menu.
    """
    return render_template('menu.html')


def build_stringlist():
    """
    Return all the strings that is used in this applet.
    """
    string_dict = {}
    string_dict['main_menu'] = _("Main Menu")
    string_dict['next_text'] = _("Next")
    string_dict['nothing_yet'] = _("There is nothing yet to do here.")
    string_dict['continue'] = _("Continue")
    string_dict['previous_text'] = _("Previous")
    return string_dict




from yasi_applets.welcome import bp
from flask import Flask, request, session, redirect, \
     url_for, render_template, flash, Blueprint
# we use this neat little trick to get config data from the main app
from flask import current_app
import gettext
import dmm.lsblk as lsblk
import dmm.timezone as timezone
import subprocess
import socket

# Set up Gettext
def set_language(LANG):
    """
    Sets language for this applet
    """
    # TODO: unhardcode this path
    translations = gettext.translation("welcome", './yasi_applets/welcome/locales',
                                       fallback=True, languages=[LANG])
    translations.install()
    _ = translations.gettext


@bp.route('/welcome',methods=['GET', 'POST', 'PUT'])
def welcome_index():
    """
    The page you'd get if you access the root of
    this app in a browser.
    """
    set_language(current_app.config['CONFIG']['settings']['language'])
    if request.method == 'POST':
        if "lang" in request.form.keys():
            current_app.config['CONFIG']['settings']['language'] = request.form["lang"]
            LANG = current_app.config['CONFIG']['settings']['language']
            set_language(LANG)
            FORMLANG = request.form["lang"]
        if "keyboard" in request.form.keys():
            print(request.form["keyboard"])
            current_app.config['CONFIG']['settings']['keyboard'] = request.form["keyboard"]
        if "timezone" in request.form.keys():
            print(request.form["timezone"])
    global lang
    lang = current_app.config['CONFIG']['settings']['language']
    keyboard = current_app.config['CONFIG']['settings']['keyboard']
    set_language(lang)
    global string_dict
    string_dict = build_stringlist()
    blkid = lsblk.list_scsi_devices()
    build_menu()
    build_summary()
    menu = current_app.config['CONFIG']['settings']['menu_order'].split(" ")
    next_step_url = menu[menu.index("welcome")+1]
    previous_step_url = menu[menu.index("welcome")-1]

    print("index is: " , menu.index("welcome"))
    if menu.index("welcome") == 0:
        previous_step_url = False
    else:
        previous_step_url = menu[menu.index("users")-1]

    power_button = '<button id="powerbutton" hx-get="/welcome/battery/button" hx-trigger="load, every 1s" hx-target="#powerbutton" class="button is-dark"> <span style="color: #ffffff;"> <i class="fa fa-question" aria-hidden="true"></i></span> &nbsp;</button>'
    internet_button = '&nbsp; <button id="internetbutton" class="button is-dark" hx-get="/welcome/internet/button" hx-trigger="load, every 1s" hx-target="#internetbutton"><span style="color: #ffffff;"> <i class="fa fa-question" aria-hidden="true"></i></span> &nbsp;</button>'
    bottom_menu = power_button + internet_button


    return render_template('welcome.html',
                           string_dict=string_dict, selected_lang=lang, keyboard=keyboard,
                           menu=current_app.config['CONFIG']['settings']['menu'],
                           menu_order=current_app.config['CONFIG']['settings']['menu_order'].split(),
                           timezones=timezone.list_timezones()[1].split("\n"),
                           previous_step=previous_step_url,
                           next_step=next_step_url,
                           bottom_menu=bottom_menu, return_path = request.path)


@bp.route('/welcome/battery',methods=['GET', 'POST', 'PUT'])
def welcome_battery():
    """
    Are we running on battery?
    How much power do we have?
    """
    if "yes" in str(subprocess.check_output("LANG=C upower -d | grep on-battery", shell=True)):
       on_battery = True
    else:
       on_battery = False
    if on_battery:
        percentage = str(subprocess.check_output('LANG=C upower -d | grep '
                         'percentage | head -1 | cut -f 2 -d":" | '
                         'sed "s/ //g"', shell=True)).strip("'b").strip("%\\n")
    else:
        percentage = False
    status = {}
    status = {"on_battery": on_battery, "percentage": percentage}
    return status


@bp.route('/welcome/battery/button',methods=['GET', 'POST', 'PUT'])
def welcome_battery_button():
    """
    Returns the html power button.
    """
    on_battery, percentage = welcome_battery()['on_battery'], welcome_battery()['percentage']
    if on_battery:
        power_button = '<button id="powerbutton hx-get="/welcome/battery/button" hx-trigger="load, every 1s" hx-target="#powerbutton" class="button is-dark"> <span style="color: #ffa45c;"> <i class="fa fa-warning" aria-hidden="true"></i></span> &nbsp; Power: Battery: ' + percentage + '%</button>'
    else:
        power_button = """<button id="powerbutton hx-get="/welcome/battery/button" hx-trigger="load, every 1s" hx-target="#powerbutton" class="button is-dark"> <span style="color: #a0ff98;"> <i class="fa fa-check" aria-hidden="true"></i></span> &nbsp;""" +  string_dict["power_text"] + "</button>"""
    return str(power_button)


@bp.route('/welcome/internet', methods=['GET'])
def has_internet():
    """
    Very rudimentary internet check.

    Check if we can resolve deb.debian.org.
    """
    try:
        socket.getaddrinfo("deb.debian.org", 80, proto=socket.IPPROTO_TCP)
        canhas = True
    except socket.gaierror:
        canhas = False
    return canhas


@bp.route('/welcome/internet/button', methods=['GET'])
def welcome_internet_button():
    """
    Draw updated button depending on internet status.
    """
    if has_internet():
        return('&nbsp; <button id="internetbutton" class="button is-dark" hx-get="/welcome/internet/button" hx-trigger="load, every 1s" hx-target="#internetbutton"><span style="color: #a0ff98;"> <i class="fa fa-check" aria-hidden="true"></i></span> &nbsp; Internet </button>')
    else:
        return('&nbsp; <button id="internetbutton" class="button is-dark" hx-get="/welcome/internet/button" hx-trigger="load, every 1s" hx-target="#internetbutton"><span style="color: #ffa45c;"> <i class="fa fa-warning" aria-hidden="true"></i></span> &nbsp; Internet </button>')


def build_stringlist():
    """
    Return all the strings that is used in this applet."
    """
    string_dict = {}
    string_dict['menu_item'] = _("Welcome")
    # In the future, these will be loaded from their own modules
    string_dict['menu_item_users'] = _("Users")
    string_dict['menu_item_disks'] = _("Disks")
    string_dict['menu_item_software'] = _("Software")
    string_dict['menu_item_summary'] = _("Summary")
    string_dict['welcome_text'] = _("Welcome! This setup program will install Debian on to your system.")
    string_dict['confirm_text'] = _("Please confirm the following details:")
    string_dict['language_text'] = _("Language:")
    string_dict['keylayout_text'] = _("Keyboard Layout:")
    string_dict['timezone_text'] = _("Time Zone:")
    string_dict['basicsettings_text'] = _("Basic Settings:")
    string_dict['power_text'] = _("Power")
    string_dict['internet_text'] = _("Internet")
    string_dict['next_text'] = _("Next")
    string_dict['back_text'] = _("Back")
    lang_dict = {}
    lang_dict['af'] = _("Afrikaans")
    lang_dict['en'] = _("English (International)")
    lang_dict['en-us'] = _("English (United States)")
    lang_dict['en-uk'] = _("English (United Kingdom)")
    lang_dict['en-za'] = _("English (South Africa)")
    string_dict['lang_list'] = lang_dict
    return string_dict


def build_menu():
    """
    Define menu items and paths.
    """
    current_app.config['CONFIG']['settings']['menu']['welcome'] = (build_stringlist()['menu_item'], "/welcome", 10)

    #TODO: temporary hack: We need an automated way to translate all menu items
    current_app.config['CONFIG']['settings']['menu']['users'] = (build_stringlist()['menu_item_users'], "/users", 20)
    current_app.config['CONFIG']['settings']['menu']['disks'] = (build_stringlist()['menu_item_disks'], "/disks", 30)
    current_app.config['CONFIG']['settings']['menu']['software'] = (build_stringlist()['menu_item_software'], "/software", 40)
    current_app.config['CONFIG']['settings']['menu']['summary'] = (build_stringlist()['menu_item_summary'], "/summary", 50)


def build_summary():
    """
    Write up a summary of what this module will do.
    """
    html = ("<b><big>Basic settings</big></b><p> Language: " + lang + "</p>")
    if not 'Summary' in current_app.config['CONFIG']:
        current_app.config['CONFIG']['Summary'] = {}
    current_app.config['CONFIG']['Summary']['welcome'] = {}
    current_app.config['CONFIG']['Summary']['welcome']['heading'] = "Basic Settings"
    print(current_app.config['CONFIG']['Summary']['welcome'])
    text = "Language: " + str(current_app.config['CONFIG']['settings']['language'])
    current_app.config['CONFIG']['Summary']['welcome']['text'] = text
    return(html)


set_language(current_app.config['CONFIG']['settings']['language'])
build_menu



from yasi_applets.disks import bp
from flask import Flask, request, session, redirect, \
     url_for, render_template, flash, Blueprint
import dmm.lsblk as lsblk
import gettext
from flask import current_app

@bp.route('/disks')
def disks_welcome():
    """
    Welcome menu for partitioning
    """
    menu = current_app.config['CONFIG']['settings']['menu_order'].split(" ")
    next_step_url = menu[menu.index("disks")+1]
    if menu[menu.index("disks")] == 1:
        previous_step_url = False
    else:
        previous_step_url = menu[menu.index("disks")-1]

    blockdevs = lsblk.list_block_devices()
    global string_dict
    string_dict = build_stringlist()
    next_step_url = False

    select_button = '<button class="button is-dark"> <span style="color: #ffa45c;"> </span> Select an option above</button>'
    bottom_menu_right = select_button


    return render_template('disks_welcome.html',
                           menu=current_app.config['CONFIG']
                                                  ['settings']['menu'],
                           menu_order=current_app.config['CONFIG']
                                                        ['settings']
                                                        ['menu_order'].split(),
                           previous_step=previous_step_url,
                           next_step=next_step_url, blockdevs=blockdevs,
                           string_dict = string_dict, 
                           bottom_menu_right = bottom_menu_right
                           )


@bp.route('/disks/choose_disk_erase')
def disks_choose_disk_erase():
    """
    Choose a disk to erase
    """
    menu = current_app.config['CONFIG']['settings']['menu_order'].split(" ")
    next_step_url = menu[menu.index("disks")+1]
    if menu[menu.index("disks")] == 1:
        previous_step_url = False
    else:
        previous_step_url = menu[menu.index("disks")-1]

    blockdevs = lsblk.list_block_devices()

    next_step_url = False
    select_button = '<button class="button is-dark"> <span style="color: #ffa45c;"> </span> Select a disk</button>'
    bottom_menu_right = select_button


    return render_template('disks_erase_menu.html',
                           menu=current_app.config['CONFIG']
                                                  ['settings']['menu'],
                           menu_order=current_app.config['CONFIG']
                                                        ['settings']
                                                        ['menu_order'].split(),
	                       previous_step = previous_step_url,
                           next_step=next_step_url, blockdevs=blockdevs,
                           bottom_menu_right = bottom_menu_right,
                           string_dict = build_stringlist()
                           )


@bp.route('/disks/choose_disk_erase_chosen/<disk>')
def disks_choose_disk_erase_confirm(disk):
    """
    Disk chosen, show before and after of disk configuration.
    """

    partitions = current_app.config['CONFIG']['recipe']['create_partitions']

    menu = current_app.config['CONFIG']['settings']['menu_order'].split(" ")
    for index, disk_part in enumerate(current_app.config['CONFIG']['recipe']['create_partitions']['command_set']):
        if "nvme" in disk or "mmc" in disk or "loop" in disk:
            partspec = "p"
        else:
            partspec = ""
        current_app.config['CONFIG']['recipe']['create_partitions']['command_set'][index]["device"] = "/dev/" + disk
        current_app.config['CONFIG']['recipe']['create_partitions']['command_set'][index]["partition"] = "/dev/" + disk + partspec + str(index+1)
    next_step_url = menu[menu.index("disks")+1]
    if menu[menu.index("disks")] == 1:
        previous_step_url = False
    else:
        previous_step_url = menu[menu.index("disks")-1]
    blockdevs = lsblk.list_block_devices()

    return render_template('disks_erase_confirmed.html',
                           menu=current_app.config['CONFIG']
                                                  ['settings']['menu'],
                           menu_order=current_app.config['CONFIG']
                                                        ['settings']
                                                        ['menu_order'].split(),
                           previous_step = previous_step_url,
                           next_step=next_step_url, blockdevs=blockdevs,
                           diskname=disk, partitions=partitions,
                           string_dict = build_stringlist()
                           )


@bp.route('/disks/advanced')
def disks():
    """
    Disks screen for the webui.
    """
    blockdevs = lsblk.list_block_devices()
    print(lsblk.list_block_devices())
    menu = current_app.config['CONFIG']['settings']['menu_order'].split(" ")
    next_step_url = menu[menu.index("disks")+1]
    if menu[menu.index("disks")] == 1:
        previous_step_url = False
    else:
        previous_step_url = menu[menu.index("disks")-1]

    add_disks_button = '&nbsp; <button class="button iis-info"> <i class="fa fa-hdd-o" aria-hidden="true"></i> &nbsp; Add Disk or Filesystem </button>'
    add_swap_button = '&nbsp; <button class="button iis-info"> <i class="fa fa-object-group" aria-hidden="true"></i> &nbsp; Swap Configuration  </button>'
    bottom_menu = add_disks_button + add_swap_button
    # no bottom menu for this release
    bottom_menu = ""
    build_summary()
    global string_dict
    string_dict = build_stringlist()
    print("SD: ", string_dict)

    return render_template('disks_advanced.html', blockdevs=blockdevs,
        menu=current_app.config['CONFIG']['settings']['menu'],
	menu_order=current_app.config['CONFIG']['settings']['menu_order'].split(),
	previous_step = previous_step_url,
	next_step=next_step_url,
	partitions=current_app.config['CONFIG']['recipe']['create_partitions']['command_set'],
	format_partitions=current_app.config['CONFIG']['recipe']['format_partitions'],
	bottom_menu=bottom_menu, string_dict=string_dict)


@bp.route('/disks/partition/<part>')
def disks_partition(part):
    """
    Partition modal for the webui partition screen.
    """
    print("Partition is: " + part)
    partition = part
    return render_template('disks_partition.html',
        menu=current_app.config['CONFIG']['settings']['menu'],
    	partition=partition,  string_dict = build_stringlist())


@bp.route('/disks/details/<disk>')
def disks_details(disk):
    """
    Show some details about a disk.
    """
    blockdevs = lsblk.list_block_devices()
    return render_template('disks_details.html', disk=disk, blockdevs=blockdevs,
                           string_dict = build_stringlist())


@bp.route('/disks/wipe_table/<disk>')
def disks_wipe_table(disk):
    """
    Create a new partition table
    """
    return render_template('disks_replace_partition.html', disk=disk,
                           string_dict = build_stringlist())


@bp.route('/disks/choose_partition_erase/')
def disk_wipe_partition():
    """
    Select a partition to erase
    """
    blockdevs = lsblk.list_block_devices()
    return render_template('disks_wipe_part_select.html', disk='',
                           blockdevs = blockdevs,
                           string_dict = build_stringlist())


def build_stringlist():
    """
    Return all the strings that is used in this applet.
    """
    string_dict = {}
    string_dict['menu_item'] = _("Disks")
    string_dict['welcome_text'] =  ("How would you like to install Debian?")
    string_dict['install_next_text'] = _("Install next to existing partition")
    string_dict['replace_part_text'] = _("Replace a partition")
    string_dict['erase_disk_text'] = _("Erase entire disk")
    string_dict['advanced_part_text'] = _("Advanced partitioning")
    string_dict['preconfigured_part_text'] = _("Preconfigured Partitioning")
    string_dict['detected_disks_text'] = _("The Following devices have been detected:")
    string_dict['confirm_disk_erase'] = _("Please confirm the disk to erase:")
    string_dict['disk_text'] = _("disk")
    string_dict['serial_text'] = _("serial")
    string_dict['select_disk_text'] = _("Select Disk")
    string_dict['partition_text'] = _("Partition")
    string_dict['type_text'] = _("Type")
    string_dict['mount_point_text'] = _("Mount Point")
    string_dict['label_text'] = _("Label")
    string_dict['size_text'] = _("Size")
    string_dict['used_text'] = _("Used")
    string_dict['disk_details_text'] = _("Disk Details")
    string_dict['wipe_disk_text'] = _("Wipe Disk")
    string_dict['new_partition_text'] = _("New Partition")
    string_dict['replicate_layout_text'] = _("Replicate Layout")
    string_dict['diagnostics_text'] = _("Diagnostics")
    string_dict['proposed_part_text'] = _("Proposed Partitioning")
    string_dict['confirm_text'] = _("Confirm")
    return string_dict


def build_menu():
    """
    Define menu items and paths.
    """
    # proper one once translations are done:
    #current_app.config['CONFIG']['settings']['menu']['welcome'] = (build_stringlist()['menu_item'], "/welcome", 10)
    current_app.config['CONFIG']['settings']['menu']['disks'] = ("Disks", "/disks", 30)


def build_summary():
    """
    Write up a summary of what this module will do.
    """
    current_app.config['CONFIG']['Summary']['disks'] = {}
    current_app.config['CONFIG']['Summary']['disks']['heading'] = "Disks and Partitions"
    current_app.config['CONFIG']['Summary']['disks']['bleh'] = current_app.config['CONFIG']['recipe']['popcon']['enable_popcon']
    current_app.config['CONFIG']['Summary']['disks']['text'] = "Pre-configured disk layout."
    return("ok?")

build_menu()



from yasi_applets.summary import bp
from flask import Flask, request, session, redirect, \
     url_for, render_template, flash, Blueprint
from flask import current_app
import yaml

@bp.route('/summary',methods=['GET', 'POST', 'PUT'])
def summary_index():
    """
    The page you'd get if you access the root of
    this app in a browser.
    """
    build_menu()
    menu = current_app.config['CONFIG']['settings']['menu_order'].split(" ")
    previous_step_url = menu[menu.index("summary")-1]
    next_step_url = "install"

    config = current_app.config['CONFIG']
    print (current_app.config['CONFIG']['Summary'])
    html = (str(current_app.config['CONFIG']['Summary']['welcome']) + "<br>" + str(current_app.config['CONFIG']['Summary']['software']))
    summary = config['Summary']

    print("Config is: ")
    print(config)
    print("YAML config is: ")
    print(yaml.dump(config))
    with open('/etc/yasi/final-install.yaml', 'w') as file:
        yaml.safe_dump(config, file, default_flow_style=False, sort_keys=False)

    return render_template('summary.html',
        menu=current_app.config['CONFIG']['settings']['menu'],
	menu_order=current_app.config['CONFIG']['settings']['menu_order'].split(),
	previous_step=previous_step_url,
	next_step=next_step_url,
	config=config,
	html=html,
	summary=summary)


def build_menu():
    """
    Define menu items and paths.
    """
    # proper one once translations are done:
    #current_app.config['CONFIG']['settings']['menu']['welcome'] = (build_stringlist()['menu_item'], "/welcome", 10)
    current_app.config['CONFIG']['settings']['menu']['summary'] = ("Summary", "/summary", 60)


build_menu()




from flask import Blueprint

bp = Blueprint('users', __name__,
               template_folder='',
               static_folder='static',
               static_url_path='/users/static')


from yasi_applets.users import routes




from yasi_applets.software import bp
from flask import Flask, request, session, redirect, \
     url_for, render_template, flash, Blueprint
from flask import current_app

@bp.route('/software')
def webui_software():
    """
    Software screen for the webui.
    """
    menu = current_app.config['CONFIG']['settings']['menu_order'].split(" ")
    next_step_url = menu[menu.index("software")+1]
    if menu[menu.index("software")] == 1:
        previous_step_url = False
    else:
        previous_step_url = menu[menu.index("software")-1]

    sources_button = '&nbsp; <button class="button iis-info"> <i class="fa fa-cloud-download" aria-hidden="true"></i> &nbsp; Edit Sources </button>'
    blends_button = '&nbsp; <button class="button iis-info"> <i class="fa fa-download" aria-hidden="true"></i> &nbsp; Install a Blend  </button>'
    bottom_menu = sources_button + blends_button
    # Leave out useless buttons until they do more
    bottom_menu = ''
    build_summary()

    return render_template('software.html',
        menu=current_app.config['CONFIG']['settings']['menu'],
	menu_order=current_app.config['CONFIG']['settings']['menu_order'].split(),
	previous_step = previous_step_url,
	next_step = next_step_url,
	bottom_menu = bottom_menu,
	popcon=current_app.config['CONFIG']['recipe']['popcon']['enable_popcon'],
	desktop=current_app.config['CONFIG']['recipe']['install_desktop_environment']['packages'])


@bp.route('/software/settings', methods=['GET', 'POST'])
def software_settings():
    """
    Receive settings for the software applet.
    """
    if request.method == 'POST':
        popcon = "popcon" in request.form
        current_app.config['CONFIG']['recipe']['popcon']['enable_popcon'] = popcon
        print(current_app.config['CONFIG']['recipe']['popcon']['enable_popcon'])
        build_summary()
        return ('', 204)


@bp.route('/software/select-desktop', methods=['GET', 'POST'])
def software_select_desktop():
    """
    Select a desktop environment for installation
    """
    if request.method == 'POST':
        desktop = request.form['software-desktop']
        print(desktop)
        current_app.config['CONFIG']['recipe']['install_desktop_environment']['packages'] = desktop
        build_summary()
        return ('', 204)


def build_menu():
    """
    Define menu items and paths.
    """
    # proper one once translations are done:
    #current_app.config['CONFIG']['settings']['menu']['welcome'] = (build_stringlist()['menu_item'], "/welcome", 10)
    current_app.config['CONFIG']['settings']['menu']['software'] = ("Software", "/software", 40)


def build_summary():
    """
    Write up a summary of what this module will do.
    """
    current_app.config['CONFIG']['Summary']['software'] = {}
    current_app.config['CONFIG']['Summary']['software']['heading'] = "Software"
    current_app.config['CONFIG']['Summary']['software']['bleh'] = current_app.config['CONFIG']['recipe']['popcon']['enable_popcon']
    current_app.config['CONFIG']['Summary']['software']['text'] = "Participate in Popularity Contest: " + str(current_app.config['CONFIG']['recipe']['popcon']['enable_popcon']) + "<br/>Desktop environment selected: " + str(current_app.config['CONFIG']['recipe']['install_desktop_environment']['packages'])
    return("ok?")


build_menu()



from flask import Blueprint

bp = Blueprint('welcome', __name__,
               template_folder='',
               static_folder='static',
               static_url_path='/welcome/static')

from yasi_applets.welcome import routes



from yasi_applets.software import bp
from flask import Flask, request, session, redirect, \
     url_for, render_template, flash, Blueprint
import gettext
# we use this neat little trick to get config data from the main app
from flask import current_app

# Set up Gettext
def set_language(LANG):
    """
    Sets language for this applet
    """
    translations = gettext.translation("users", './yasi_applets/users/locales',
                                       fallback=True, languages=[LANG])
    translations.install()
    _ = translations.gettext

set_language('')


@bp.route('/users')
def users_root():
    """
    Users screen for the webui.
    """
    set_language(current_app.config['CONFIG']['settings']['language'])
    string_dict = build_stringlist()
    build_menu()
    menu = current_app.config['CONFIG']['settings']['menu_order'].split(" ")
    next_step_url = menu[menu.index("users")+1]
    if menu[menu.index("users")] == 1:
        previous_step_url = False
    else:
        previous_step_url = menu[menu.index("users")-1]

    root_button = '&nbsp; <button class="button not-allowed is-light"> <i class="fa fa-user" aria-hidden="true"></i> &nbsp; Setup root user </button>'
    ldap_button = '&nbsp; <button class="button is-light"> <i class="fa fa-address-card" aria-hidden="true"></i> &nbsp; Connect LDAP </button>'
    ad_button = '&nbsp; <button class="button is-light"> <i class="fa fa-address-card" aria-hidden="true"></i> &nbsp; Connect AD </button>'
    csv_button = '&nbsp; <button class="button is-light"> <i class="fa fa-users" aria-hidden="true"></i> &nbsp; Import CSV </button>'
    bottom_menu = root_button + ldap_button + ad_button + csv_button
    bottom_menu = ""

    initial_user = current_app.config['CONFIG']['recipe']['users']['users'][0]

    build_summary()

    return render_template('users.html', string_dict=string_dict,
                           menu=current_app.config['CONFIG']['settings']['menu'],
                           menu_order=current_app.config['CONFIG']['settings']['menu_order'].split(),
                           previous_step=previous_step_url,
                           next_step=next_step_url,
                           bottom_menu=bottom_menu,
                           initial_user=initial_user)


@bp.route('/users/user-check/', methods=['GET', 'POST', 'PUT'])
def check_user():
    if request.method == 'POST':
        username = request.form["username"]
        fullname = request.form["fullname"]
        current_app.config['CONFIG']['recipe']['users']['users'][0]['username'] = username
        current_app.config['CONFIG']['recipe']['users']['users'][0]['fullname'] = fullname
        if username in ["root", "games"]:
            return('<p class="has-text-danger"><b>That username is not available</b> </p>')
        else:
            return('<p class="has-text-success"><b>That username is available</b> </p>')
        build_summary()
    return("")


@bp.route('/users/password-check/', methods=['GET', 'POST', 'PUT'])
def check_password():
    if request.method == 'POST':
        password = request.form["password"]
        password_confirm = request.form["password_confirm"]
        current_app.config['CONFIG']['recipe']['users']['users'][0]['password'] = password
        current_app.config['CONFIG']['recipe']['users']['users'][0]['password_confirm'] = password_confirm
        if password != password_confirm:
            return('<p class="has-text-danger"><b>Passwords do not match</b> </p>')
        else:
            return('<p class="has-text-success"><b>Passwords match</b> </p>')
        build_summary()
    return("")


def build_stringlist():
    """
    Return all the strings that is used in this applet.
    """
    string_dict = {}
    string_dict['menu_item'] = _("Users")
    string_dict['initial_user_text'] = _("Let's set up an initial user.")
    string_dict['full_name'] = _("Full Name")
    string_dict['user_name'] = _("Username")
    string_dict['user_name_hint'] = _("One word, all lowercase")
    string_dict['user_name_available'] = _("This username is available")
    string_dict['user_name_not_available'] = _("This username is not available")
    string_dict['user_name_reserved'] = _("This username is reserved by the system")
    string_dict['user_name_characters'] = _("The username must be one word, lowercase, with no special characters")
    string_dict['password'] = _("Password")
    string_dict['password_confirm'] = _("Password (confirm)")
    string_dict['password_nomatch'] = _("These passwords do now match")
    string_dict['password_tooshort'] = _("This password is too short")
    string_dict['next_text'] = _("Next")
    string_dict['back_text'] = _("Back")
    return string_dict


def build_menu():
    """
    Define menu items and paths.
    """
    current_app.config['CONFIG']['settings']['menu']['users'] = (build_stringlist()['menu_item'], "/users", 20)


def build_summary():
    """
    Write up a summary of what this module will do.
    """
    current_app.config['CONFIG']['Summary']['users'] = {}
    current_app.config['CONFIG']['Summary']['users']['heading'] = "Users and Identity"
    current_app.config['CONFIG']['Summary']['users']['settings'] = current_app.config['CONFIG']['recipe']['users']['users']
    current_app.config['CONFIG']['Summary']['users']['text'] = "Add primary user with the username: " + str(current_app.config['CONFIG']['recipe']['users']['users'][0]['username'])
    return("ok?")

build_menu()
